"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CloudRun = void 0;
const tslib_1 = require("tslib");
const pulumi = require("@pulumi/pulumi");
const gcp = require("@pulumi/gcp");
const service_account_1 = require("./service-account");
const env_vars_1 = require("../libs/env-vars");
const utils_1 = require("../libs/utils");
class CloudRun extends pulumi.ComponentResource {
    constructor(name, args, opts = {}) {
        super('wordpress:cloudrun', name, {}, opts);
        let { envs = [], databaseSettings, containerConcurrency = 20, maxScale = 5, minScale = 0, domain } = args, config = tslib_1.__rest(args, ["envs", "databaseSettings", "containerConcurrency", "maxScale", "minScale", "domain"]);
        this.storageBucket = new gcp.storage.Bucket(name, {
            location: 'eu',
            cors: [
                {
                    methods: ['*'],
                    origins: ['*'],
                },
            ],
        }, { parent: this });
        this.serviceAccount = new service_account_1.ServiceAccount(name, {
            accountId: utils_1.escapeName(`sa-${name}`),
            roles: ['roles/cloudsql.client'],
        }, { parent: this });
        this.storageBucketIamMember = [
            new gcp.storage.BucketIAMMember(`${name}-viewer`, {
                bucket: this.storageBucket.name,
                member: 'allUsers',
                role: 'roles/storage.objectViewer',
            }, { parent: this }),
            new gcp.storage.BucketIAMMember(`${name}-service`, {
                bucket: this.storageBucket.name,
                member: this.serviceAccount.account.email.apply(s => `serviceAccount:${s}`),
                role: 'roles/storage.objectAdmin',
            }, { parent: this }),
        ];
        const annotations = {
            'autoscaling.knative.dev/maxScale': String(maxScale),
            'autoscaling.knative.dev/minScale': String(minScale),
        };
        if (databaseSettings) {
            annotations['run.googleapis.com/cloudsql-instances'] =
                databaseSettings.connectionName;
            envs.push({
                name: 'CLOUDSQL_INSTANCE',
                value: databaseSettings.connectionName,
            });
            envs.push(...env_vars_1.createEnvironmentVariables({
                instance: databaseSettings,
            }));
        }
        this.service = new gcp.cloudrun.Service(name, {
            name,
            autogenerateRevisionName: true,
            location: config.location,
            template: {
                metadata: {
                    annotations,
                },
                spec: {
                    containerConcurrency,
                    serviceAccountName: this.serviceAccount.account.email,
                    containers: [
                        {
                            image: config.image,
                            envs: [
                                {
                                    name: 'STORAGE_BUCKET',
                                    value: this.storageBucket.name,
                                },
                                {
                                    name: 'GCP_SERVICE_ACCOUNT',
                                    value: this.serviceAccount.key.privateKey,
                                },
                                ...envs,
                            ],
                        },
                    ],
                },
            },
        }, { parent: this });
        if (domain) {
            const project = gcp.organizations.getProject(undefined, { parent: this });
            this.domainMapping = new gcp.cloudrun.DomainMapping(name, {
                location: config.location,
                metadata: {
                    namespace: project.then(p => p.projectId),
                },
                name: domain,
                spec: { routeName: this.service.name },
            }, { parent: this });
        }
        this.cloudRunIamMember = new gcp.cloudrun.IamMember(name, {
            service: this.service.name,
            location: config.location,
            role: 'roles/run.invoker',
            member: 'allUsers',
        }, { parent: this });
    }
}
exports.CloudRun = CloudRun;
